package cn.com.ty.lift.business.library.service;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.PlatformCompanyLiftRelevance;
import cn.com.ty.lift.business.library.dao.entity.model.LiftExtensionRequest;
import cn.com.ty.lift.business.library.dao.entity.model.LiftRequest;
import cn.com.ty.lift.business.library.dao.entity.model.LiftResponse;
import cn.com.ty.lift.business.library.dao.mapper.LiftMapper;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceCompany;
import cn.com.ty.lift.business.maintenance.service.MaintenanceService;
import cn.com.ty.lift.common.base.ExportRequest;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.ty.lift.common.export.ExportUtils;
import cn.com.ty.lift.common.utils.DateUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author bieao
 * @date 2019/11/27
 * @description 电梯业务层
 */
@Service
public class LiftService {

    @Resource
    private LiftMapper liftMapper;

    @Resource
    private PlatformCompanyLiftRelevanceService platformService;

    @Resource
    private ProjectLiftRelevanceService projectRelevanceService;

    @Resource
    private MaintenanceService maintenanceService;

    private Map<String, String> paramMap = new HashMap<String, String>() {{
        put("liftCode", "电梯号");
        put("registrationCode", "注册代码");
        put("liftBrand", "电梯品牌");
        put("liftType", "电梯类型");
        put("devicePosition", "设备使用地点");
    }};

    /**
     * @param request 电梯列表查询条件
     * @return IPage<LiftResponse> 电梯列表分页结果
     * @description 条件查询电梯档案列表
     * @date 2019/11/27 2:10 PM
     */
    public IPage<LiftResponse> list(LiftRequest request) {
        IPage<LiftResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return liftMapper.findByCondition(page, request);
    }

    /**
     * @param request 电梯列表查询条件
     * @return IPage<LiftResponse> 项目下电梯列表
     * @description 查询项目下电梯列表
     * @date 2019/12/17 11:04 AM
     */
    public IPage<LiftResponse> findLiftListByProjectId(LiftRequest request) {
        IPage<LiftResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return liftMapper.findLiftListByProjectId(page, request);
    }

    /**
     * @param id 电梯id
     * @return Lift 电梯详情
     * @description 查询电梯详情
     * @date 2019/12/6 10:51 AM
     */
    public Lift detail(Long id) {
        return liftMapper.selectById(id);
    }


    /**
     * @param mtCompanyId 公司id
     * @param code        注册代码
     * @param liftId      电梯id
     * @return RestResponse 判断结果
     * @description 新增/修改电梯前置判断条件
     * @date 2019/12/20 2:31 PM
     */
    public RestResponse preJudgment(Long mtCompanyId, String code, String liftId) {
        //查询电梯表是否有该电梯
        Long id = findLiftExist(code, liftId);
        if (ObjectUtil.isNotEmpty(id) && ObjectUtil.isEmpty(liftId)) {
            Map<String, Object> paramMap = new HashMap<>(2);
            paramMap.put("id", id);
            paramMap.put("mtCompanyId", mtCompanyId);
            //查询该公司下是否有此电梯
            boolean result = platformService.findCompanyLiftExist(paramMap);
            if (result) {
                return RestResponse.fail(MessageUtils.get("msg.lift.exist"));
            } else {
                Lift detail = detail(id);
                return RestResponse.success(detail, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.lift.company.exist"));
            }
        } else if (ObjectUtil.isNotEmpty(id) && ObjectUtil.isNotEmpty(liftId)) {
            return RestResponse.fail(MessageUtils.get("msg.lift.library.exist"));
        } else {
            return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.pre.judge"));
        }
    }

    /**
     * @param mtCompanyId 维保公司id
     * @return 比较结果
     * @description 判断当前公司台量是否大于企业设置的台量
     * @date 2019/12/12 5:27 PM
     */
    public boolean judge(Long mtCompanyId) {
        //查询维保公司设置的最大电梯台量
        MaintenanceCompany company = maintenanceService.selectById(mtCompanyId);
        Integer limitedNum = company.getLimitedNum();
        //获取该公司企业电梯关联表的实际电梯台量
        Integer currentNum = platformService.countLiftNum(mtCompanyId);
        return currentNum >= limitedNum;
    }

    /**
     * @param registrationCode 电梯注册代码
     * @param liftId 电梯id
     * @return 电梯id
     * @description 查询电梯是否存在
     * @date 2019/12/20 11:04 AM
     */
    public Long findLiftExist(String registrationCode, String liftId) {
        return liftMapper.findLiftExist(registrationCode, liftId);
    }

    /**
     * @param request 新增电梯数据项
     * @return RestResponse 状态码和返回消息
     * @description 新增电梯
     * @date 2019/11/27 2:22 PM
     */
    @Transactional
    public RestResponse add(LiftExtensionRequest request) {
        //获取新增标记 (0:新增,1:查询后带入)
        String mark = request.getMark();
        //公司id
        Long mtCompanyId = request.getMtCompanyId();
        //电梯id
        Long id;
        /*if (judge(mtCompanyId))
        return RestResponse.fail(ApiConstants.RESULT_ERROR, MessageUtils.get("msg.limit.num"));*/
        RestResponse response = preJudgment(mtCompanyId, request.getCode(), null);
        if (ApiConstants.RESULT_ERROR.equals(response.getMessage())) return response;
        //新增标记 (0:新增,1:查询后带入)
        if ("0".equals(mark)) {
            Lift lift = request.getLift();
            //按当前日期时间戳自动生成电梯编号
            String liftCode = ApiConstants.STRAIGHT_LADDER + DateUtils.generateCode();
            if (CommonEnum.LiftType.STAIRCASE.getCode().equals(Objects.toString(lift.getLiftType()))) {
                lift.setLiftCode(ApiConstants.STAIRCASE + liftCode);
            }
            //新增电梯表
            int liftResult = liftMapper.insert(lift);
            if (liftResult == 0) {
                return RestResponse.fail(MessageUtils.get("msg.add.fail"));
            }
            id = lift.getId();
        } else {
            id = Long.parseLong(request.getId());
        }
        //新增电梯来源
        String source = request.getSource();
        switch (source) {
            //电梯管理入口
            case "0":
                //新增企业电梯关联表
                PlatformCompanyLiftRelevance ret = platformService.save(id, mtCompanyId);
                if (ret == null) {
                    return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                }
                //新增平台电梯关联表
                PlatformCompanyLiftRelevance liftRelevance = platformService.save(id, ApiConstants.PLATFORM_COMPANY_ID);
                if (liftRelevance == null) {
                    return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                }
                break;
            //项目管理入口
            case "1":
                //新增企业电梯关联表
                PlatformCompanyLiftRelevance companyLiftEntry = platformService.save(id, mtCompanyId);
                if (companyLiftEntry == null) {
                    return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                }
                //新增平台电梯关联表
                PlatformCompanyLiftRelevance relevance = platformService.save(id, ApiConstants.PLATFORM_COMPANY_ID);
                if (relevance == null) {
                    return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                }
                //新增项目电梯关联表
                boolean result = projectRelevanceService.save(request, companyLiftEntry.getId(), request.getWorkerId());
                if (!result) {
                    return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                }
                break;
            default:
                break;
        }
        return RestResponse.success(null, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.add.success"));
    }

    /**
     * @param lift 修改电梯数据项
     * @return RestResponse 状态码和返回消息
     * @description 修改电梯
     * @date 2019/11/27 2:22 PM
     */
    public RestResponse modify(Lift lift) {
        String liftId = StrUtil.EMPTY;
        if (ObjectUtil.isNotEmpty(lift)) {
            liftId = lift.getId().toString();
        }
        RestResponse response = preJudgment(null, lift.getRegistrationCode(), liftId);
        if (ApiConstants.RESULT_ERROR.equals(response.getMessage())) return response;
        int result = liftMapper.updateById(lift);
        if (result > 0) {
            return RestResponse.success();
        }
        return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
    }

    /**
     * @param request 导出电梯数据列表
     * @return RestResponse 状态码和返回消息
     * @description 导出电梯
     * @date 2019/11/27 2:22 PM
     */
    public void export(ExportRequest request) {
        List<Lift> liftList = liftMapper.selectBatchIds(request.getIds());
        ExportUtils utils = new ExportUtils();
        utils.export(liftList, paramMap, request.getExportPath());
    }
}
